/*******************************************************************************
        A PROGRAM TO CONTROL CCD CAMERA - HYUNG-MIN PARK(hmpark@kasi.re.kr)

	  
	  DESCRIPTION : 
	  				THE FILE INCLUDE FUNCTIONS TO OPERATE CCD CAMERA.
					THIS FILE CONSISTS OF TWO PARTS, ONE IS CCD CONTROL,
					ANOTHER IS VIEWER.

			
	*REVISION HISTORY
		JAN 10 2006: ADD FUNCTIONS - DISPLAY BITMAP()
		JAN 19 2006: ADD FUNCTIONS - OPEN BITMAP()
		JAN 22 2006: FIRST SUCCESS TO OPERATE CCD CAMERA - TURN ON/DISPLAY IMAGE
		JAN 29 2006: REALIZE SOME FUNCTIONS 
					- OPEN, GET CCD INFORMAITON, SET, CLOSE, DISPLAY
					- FLIP IMAGE(UPSIDE DOWN)
		FEB 14 2006: START TO WRITE FITS READ FUNCTION
		FEB 21 2006: SAVE THE DATA AS FITS FORMAT
					-INCLUDE BYTE CHANGE, BYTE SCALE, FLIP
					-USE TIME INFORMATIONS AS FITS FILE NAME
		MAR 12 2006: START TO REALIZE SEQUENTIAL OBSERVATION BY USING CVI TIMER
		MAR 14 2006: ADD SOME FUNCTIONS TO TAKEIMAGE()
		MAR 19 2006: COMPLETE SEQUENTIAL OBSERVATION BY USING TIMER
		MAR 26 2006: ADD CCD COOLING FUNCTION
		APR 23 2006: ADD BINNING FUNCTION
					MODIFY TWO FUNCTIONS - DISPALYIMAGE, TAKEIMAGE
		MAY 02 2006: REALIZE KINETIC MODE OF CCD
		MAY 15 2006: MODIFY TAKEIMAGE(), TAKEIMAGE() AND SAVEFITS()
					-ADD FRAMENUMBER, 
		MAY 21 2006: ADD SHUTTER CONTROL FUNCTION
		MAY 25 2006: REALIZE FRAME TRANSFER MODE
		JUN 04 2006: REALIZE REAL TIME OPERATION BY USING THREAD FROM STANDARD C CODE
		AUG 30 2006: SUCCESS TO DISPLAY DATA FROM CCD CAMERA BY USING NI-VISION LIBRARY
		NOV 15 2006: MODIFY SOME FUNCTIONS - BYTEDISPLAY(MIN, MAX)
		NOV 20 2006: ADD 3D DISPLAY FUNCTION
		NOV 23 2006: SUCCESS TO DISPLAY DATA ON PANNEL PICTURE COMPONENT(CVI FUNCTION)
		NOV 28 2006: MODIFY SAVEFITS FUNCTION(3D DATA SAVE)
		JAN 16 2008: REALIZE VIEWER FUNCTION
					DISPLAY BMP AND FITS FORMAT
					2D(OPENFITS()),3D(OPENFITS1())
					CONSIDER 2 CAMERA CONNECTION
		MAR 19 2008: ADD DISPLAY HISTOGRAM
					 ADD READ MOUSE POSITION BY USING TIMER
		MAY 20 2008: MODIFY PROGRAM - CONSIDER PROCESS SPEED
		JUL 16 2008: COMPLETE INCREASE PROCESSING SPEED
		JUL 28 2008: MODIFY GUI- INPUT WINDOW
					ADD SCANNER CONTROL PROGRAM
		AUG 02 2008: DISPLAY 3D DATA ON PICTURE RING COMPONENT
		AUG 04 2008: INTEGRATE TWO FUNCTIONS OPENFITS() AND OPENFITS1()
		AUG 06 2008: MAKE VIDEO MODE BY USING CVI THREAD FUNCTION
		AUG 08 2008: READ FITS FILE AS 2D OR 3D BY USING SELECT BUTTON
		AUG 20 2008: MODIFY HISTOGRAM - ADD REFRESH FUNCTION
		OCT 22 2008: CHANGE SDK VERSION - 2.78
		OCT 28 2008: RENAME FILENAME
		NOV 06 2008: INTEGRATION PARAMETERS BETWEEN CCD AND SCANNER(SUPERPOSITION)
		FEB 09 2009: ADD DATA PATH BY USING ANDOR CCD FUNCTION
		MAR 11 2009: MODIFY DISPLAYIMAGE()
		APR 01 2009: COMPLETE FITS DISPLAY
		JUL 01 2009: INTEGRATE CCD_SCANNER PROGRAM AND GRATING/CCD CONTROL PROGRAM
		SEP 15 2009: MODIFY PROGRAM - SCANNER THREAD, MENU SIMPLIFY, 
		SEP 22 2009: DEBUG HISTOGRAM FUNCTION
					ADD LINE PROFILE DISPLAY FUNCTION
		NOV 11 2009: CHANGE THE ROUTINE - USE NASA CFITSIO FITS LIBRARY
					REDUCE FITS SAVE TIME FROM ABOUT ~13 SEC TO ABOUT ~3 SEC
		DEC 02 2009: MODIFY SOME FUNCTIONS - CONSIDER 2 CAMERA USING
					 SETCAMERAPARAM() - ADD ONE PARAMETER - CAMERA HANDLE
					 DATADUMP() - ADD 1 PARAMETER - G_PRAWDATA2
					 
********************************************************************************/




#include <ansi_c.h>
#include "CCD.h"
#include "Atmcd32d.h"
#include <analysis.h>
#include "FISS.h"
#include "fitsio.h"
#include "idltools.h"
#include <utility.h>
#include "CCDControl.h"
#include "graphic.h"
#include "grating_focus.h"
#include "scanner.h"
#include "TalkClientDLL.h"
//Define glabal parameter
//
//		nCameraHandle1	: Handle value of CCD1
//		nCameraHandle2	: Handle value of CCD2
//		nUnknownCameraHandle: unknown handle to identify camera handle
//
//		ccd_HBin 		: Binning Ratio of X-axis(horizontal) E.X)1,2,4,8..ETC.
//		ccd_VBin		: Binning Ratio of Y-azis(vertical)   E.X)1,2,4,8.. ETC.
//		g_nXPixels 		: CCD pixel size of horizontal axis
//		g_nYPixels 		: CCD pixel size of vertical axis
//		nhDim			: ACTUAL HORIZONTAL SIZE TO SAVE DATA ON CCD CHIP 
//		nvDim			: ACTUAL VERTICAL SIZE TO SAVE DATA ON CCD CHIP
//
//		g_nFrameNumber1	: Number of images to take image
//		ccd_Gain		: Gain Value to apply CCD
//
//		g_pRawData		: The pointer to allocate data from Camera1
//		g_pRawData2		: The pointer to allocate data from Camera2
//
//		DataSource		: PARAMETER TO DISPLAY DATA FROM CCD OR FROM FITS
//
//------------------------------



AndorCapabilities caps;                 // AndorCapabilities structure  
int			errorValue;
BOOL 		errorFlag;

char 		ccd_path[256];					
int 		g_nXPixels1,g_nYPixels1;			//CCD , ȼ
int			g_nXPixels2,g_nYPixels2;
int 		nhDim, nvDim;
int			nhDim1,nvDim1,nhDim2,nvDim2;


extern long nCameraHandle1, nCameraHandle2;
extern int 	g_nFrameNumber1;
extern int	ccd_HBin, ccd_VBin, ccd_Gain; 
extern int  temp_thread;
extern	LARGE_INTEGER gliFreq, gliStart, gliEnd;  
extern float *flatdata1;
extern float *flatcube1;

//extern char data_directory[150];
extern WvSet Data[50];
extern int 	fiss_take_mode; 
extern float	scanpos;
extern int  Scanner_InPosition;   
extern int Run_thread_stop; 
extern int TabPanelHandle;	
extern int	m_solarxpos,m_solarypos, m_ringvalue;
extern char m_arnumber[50], m_observername[250];

WORD* g_pRawData	= NULL;					//CCD ٷ  ͸ ϴ 
//double*	hist_data	= NULL; 				//CCD   ׷   
WORD* BiasDark1		= NULL;

//WORD* BiasDarkTemp1		= NULL;
//WORD* BiasDarkTemp2		= NULL;


//need to display image
char 		colortable[256*4];  //modified 
int			plotHandle;
int			profileplot;		//to plot spectrum line profile

extern int mainPanel;						//FISS г Ķ
extern int FISSpanelSetting; 
extern int DataSource;						//need to distinguish the data by Fits file and or by CCD
extern int GraphicPanel;											//ȭ鿡  ǥ    CCD  ,
											//FitsȭϿ   Ǵϴ .


//The parameter include Fits File
BYTE* pFitsData		= NULL;					//Fits  ͸   ϴ 
unsigned short* pusFitsData;  				//fits ȭϿ о ϸ  ؼ ȯ  .
											//pFitsData  ϹǷ ޸𸮰    ȴ().
//BYTE* 			planeData;   
WORD*			planeDataCCD;

FITSHEADER	FitsHeader;						//Fitsȭ о ϴ ü   

int CCD_status=-1;

float stdev,mean;



////scanner parameter-need to save fits header
extern int Scanner_status;
extern int scan_mode;
extern float scan_duration, scan_size, scan_speed, scan_stroke; 
extern int  Scanner_InPosition;
extern float 	g_fExpTime1, g_fExpTime2;
extern char entire_directory[250];
//initialize camera
void OpenCamera(long camera)
{
	//AndorCapabilities caps;                 // AndorCapabilities structure   
	int		i, c;								//need to make colortable
	char information[1024];

	caps.ulSize = sizeof(AndorCapabilities);
	errorFlag=FALSE;
	
	
	
	//۾  ְ ִ κ
	GetCurrentDirectory(256,ccd_path);// Look in current working directory

	if(camera==nCameraHandle1){
				//Turn on CCD system
		errorValue=Initialize(ccd_path);  // Initialize driver in current directory
		if (errorValue!=DRV_SUCCESS)
		{
			MessagePopup("Error","Initialize Error");
			errorFlag=TRUE;
		}	
		errorValue=GetDetector(&g_nXPixels1, &g_nYPixels1); 
	}
	if(camera==nCameraHandle2){
		errorValue=Lan_Initialize(&g_nXPixels2, &g_nYPixels2);
	  	  if(errorValue!=DRV_SUCCESS)
			{
   	  		 MessagePopup("Error","Fail to initialize : Camera2");
   			 }	
	}

	// ȭ鿡 ֱ  ÷̺ 
	c=0;
	for(i=0;i<256*4;i+=4)
	{
		colortable[i]=c;
		colortable[i+1]=c;
		colortable[i+2]=c;
		colortable[i+3]=0;
		c++;
	}
	
	
	CCD_status=1;
	if(camera==nCameraHandle1)GetCameraInfo(information);
	else if(camera==nCameraHandle2)Lan_GetCameraInfo();
	//MessagePopup("Information",information);
}





//α׷ ϴ   ʿ䰡  Լ
// ϵ  ⺻ Ȯϴ Լ
//Get hardware information from CCD and Frame Grabber Card
void GetCameraInfo(char* array)
{
	//AndorCapabilities 	caps;

	char    		ccd_model[32];              // headmodel    
	char 			buffer[100];
	
	int     		VSNumber;// 	= 0;                 // Vertical Speed Index
	int     		HSNumber;// 	= 0;                 // Horizontal Speed Index	
	unsigned int 	PCB, Flex, dum1,dum2,dum3,dum4;		// Information of hardware version
	unsigned int 	eprom, cofFile, vxdRev,vxdVer,dllRev,dllVer;
	int 			minTemp, maxTemp;				//get (min,max)temperature
	int 			XPixels, YPixels;
	float 			fSTemp;
	float 			fSpeed;
	int 			index;
	int 			iSpeed;
	float 			speed;
	int 			lowGain, highGain,curgain;
	int				ampgain;
	
	int				nAD,iAD,ADNumber;
	
	float			STemp;
	

	caps.ulSize = sizeof(AndorCapabilities);  
	
    // Get camera capabilities
    errorValue=GetCapabilities(&caps);
	if(errorValue!=DRV_SUCCESS)
	{
		MessagePopup("Error","Get Andor Capabilities information Error");
		errorFlag=TRUE;
	}	
	

    // Get Head Model
    errorValue=GetHeadModel(ccd_model);
    if(errorValue!=DRV_SUCCESS)
	{
      MessagePopup("Error","Get Head Model information Error");
    }
	sprintf(array,"Head Model : %s\n",ccd_model);	

    // Get detector information
    errorValue=GetDetector(&XPixels, &YPixels); 	
    if(errorValue!=DRV_SUCCESS)
	{
      MessagePopup("Error","Get Detector information of Camera1 Error.");
    }
	sprintf(buffer, "Size of CCD : %d X %d pixels\n", XPixels, YPixels);
	strcat(array, buffer);	
 

	//
	// Set Vertical speed to max -> g_nVSNumber
	//
	STemp = 0.0;
	VSNumber = 0;
	GetNumberVSSpeeds(&index);    //index=5, what?
	

    for(iSpeed=0; iSpeed<index; iSpeed++)
	{
		GetVSSpeed(iSpeed, &speed);
		if (speed > STemp)
		{
			STemp = speed;
			VSNumber = iSpeed;
		}
    }				  /* VSNumber is 4*/
	
	sprintf(buffer, "VSNumber is %d\n", VSNumber);
	strcat(array, buffer);			
	
	GetVSSpeed(VSNumber, &speed); 	
	sprintf(buffer, "Vertical Speed : %g microseconds per pixel shift\n",speed);
	strcat(array, buffer);	
	

	
	STemp = 0.0;
	HSNumber = 0;
	ADNumber = 0;
	
	errorValue=GetNumberADChannels(&nAD);
	if(errorValue!=DRV_SUCCESS)
		MessagePopup("Error","Get number AD Channel Error\n");
	else
	{
		for(iAD=0;iAD<nAD;iAD++)
		{
			GetNumberHSSpeeds(iAD,0,&index);
			for(iSpeed=0;iSpeed<index;iSpeed++)
			{
				GetHSSpeed(iAD,0,iSpeed,&speed);
				if(speed>STemp)
				{
					STemp=speed;
					HSNumber=iSpeed;
					ADNumber=iAD;
				}
			}
		}
		
	}
	
	sprintf(buffer,"ADNumber is %d\n",ADNumber);
	strcat(array,buffer);
	sprintf(buffer,"HSNumber is %d\n",HSNumber);
	strcat(array,buffer);
	
	
	
 //   GetNumberHSSpeeds(0, 0, &nIndex);
//	
//	for(i=0; i<nIndex; i++)
//	{
//		GetHSSpeed(0, 0, i, &fSpeed);    //ad channel is 0, electron multipication(0)
//		if(fSpeed > fSTemp)
//		{
//			fSTemp = fSpeed;
//			HSNumber = i;
//		}
//	
  //  }				/*HSNumber is 0*/	

   


	
	GetHSSpeed(0,0,HSNumber, &speed);
	if(caps.ulCameraType==1)
		sprintf(buffer, "Horizontal Speed : %g MHz\n", speed);
	else
		sprintf(buffer, "Horizontal Speed : %g microseconds per pixel shift\n", speed);
	strcat(array, buffer);
	
	GetHardwareVersion(&PCB,&Flex,&dum1,&dum2,&dum3,&dum4);
	sprintf(buffer,"Plug-in card version : %.2f\n" ,(double)PCB);
	strcat(array,buffer);
	sprintf(buffer, "Flex 10K file version : %.1f\n", (double)Flex); 
	strcat(array,buffer);
	sprintf(buffer, "Camera FirmWare Version :%.2f\n", (double)dum3);
	strcat(array,buffer);
	sprintf(buffer, "Cam Firmware Build No : %.2f\n", (double)dum4);
	strcat(array,buffer);
		
	

	GetSoftwareVersion(&eprom,&cofFile,&vxdRev,&vxdVer,&dllRev,&dllVer);
	sprintf(buffer,"EPROM_ver : %.1f\n",(double)eprom);
	strcat(array,buffer);
	sprintf(buffer, "COF_file_ver : %.1f\n", (double)cofFile);
	strcat(array,buffer);
	sprintf(buffer,  "Driver_revision_num : %.1f\n", (double)vxdRev);
	strcat(array,buffer);
	sprintf(buffer,  "Driver_ver : %.1f\n", (double)vxdVer);
	strcat(array,buffer);
	sprintf(buffer,"DLL_revision_number : %.1f\n", (double)dllRev);
	strcat(array,buffer);
	sprintf(buffer, "DLL_ver : %.1f\n",(double)dllVer); 
	strcat(array,buffer);
	
	GetTemperatureRange(&minTemp,&maxTemp);
	sprintf(buffer,"Temperature range : From %d C to %d C\n", minTemp, maxTemp);
	strcat(array,buffer);
	
	GetEMGainRange(&lowGain, &highGain);
	sprintf(buffer, "EM Gain range : From %d to %d\n", lowGain, highGain);
	strcat(array,buffer);
	GetEMCCDGain(&curgain);
	sprintf(buffer,"Current EMGain value :%d\n", curgain);
	strcat(array,buffer);
	
	GetNumberPreAmpGains(&ampgain);
	sprintf(buffer, "pre amp gain :%d\n", ampgain);
	strcat(array, buffer);
	
	if(caps.ulTriggerModes & AC_TRIGGERMODE_CONTINUOUS)
		strcat(array,"Correct Hardware for Continuous Mode\n");
	else
		strcat(array,"InCorrect HardWare for Continuous Mode\n");
	
	errorValue=IsTriggerModeAvailable(10);
	if(errorValue==DRV_SUCCESS)
		strcat(array,"Software Trigger Mode Available\n");
	else
		strcat(array,"Not Valid Mode for Software Trigger\n");
	
	//GetNumberADChannels(&nIndex);
	//printf("ADChannel Index:%d\n",nIndex);	
	
		
}




//ī޶    ߿ κ.
void SetCameraParam(int acquisition, int readmode, int trigger, int gain, int HBin, int VBin, float exptime, int shutter, int fan, int number, long camera)
{
	int 	trig;
	int 	nStatus;
	char    kkk[256];
	float 	fAccumTime, cycletime;  //get accumate time, cycle time;

	int ttl, openclose;   //associate with shutter setting
	
	int hStart, vStart, hEnd, vEnd;
	int n;
	float nn;
	
	int framesize;
	
	
	///////////////////////////////////////
	//this part come from TakeImage function
	///////////////////////////////////////
	
	int nBytePerPixel 	= 2;     //because the dynamic range of CCD Camera is 14bits (need to 16bit : 2byte)
	int nByteSize;
	int nPixelSize;
	
	int	g_nVSNumber;
	int g_nHSNumber = 0;

	hStart=1;							//start horizontal pixel position on CCD chip
	vStart=1;							//start vertical pixel position on CCD chip
	

	if(camera==nCameraHandle1)
	{
		if(g_nXPixels1>1000)  //dv885
		{
			g_nVSNumber = 3;		//0; //the value determinid is 3, in case of dv885	
			hEnd=g_nXPixels1;
			vEnd=g_nYPixels1-2;	
			nhDim=((hEnd-hStart+1)/HBin)/2;			//actual horizontal size on CCD chip
			nvDim=((vEnd-vStart+1)/VBin)/2;			//actual vertical size on CCD chip
//			vEnd=g_nYPixels1;	
//			nhDim=((hEnd-hStart+1)/HBin);			//actual horizontal size on CCD chip
//			nvDim=((vEnd-vStart+1)/VBin);			//actual vertical size on CCD chip						
			nhDim1=nhDim;
			nvDim1=nvDim;
		}
		else				 //dv887
		{
			g_nVSNumber = 4;		//0; //the value determinid is 4, in case of dv887
			hEnd=g_nXPixels1;
			vEnd=g_nYPixels1;
			nhDim=(hEnd-hStart+1)/HBin;			//actual horizontal size on CCD chip
			nvDim=(vEnd-vStart+1)/VBin;			//actual vertical size on CCD chip			
			nhDim1=nhDim;
			nvDim1=nvDim;
		}
		framesize=nhDim1*nvDim1;
	}	
	
	if(camera==nCameraHandle2) 
	{
		
	}
		

	if(number > 0)			//for Single Mode
	{
		nPixelSize	= 	framesize * number;
	}

	
	if(number == 0)		//for video mode
	{
		nPixelSize	=	framesize;
	}
	
	nByteSize	=	nPixelSize * nBytePerPixel;
	
	
	
	
	if(camera == nCameraHandle1)
	{
		g_pRawData = (WORD*)malloc(nByteSize);	  //allocate the memory to store image
		memset(g_pRawData,0,nByteSize);
		if(g_pRawData==NULL)
		{
			printf("Not Allocate the memory for g_pRawData\n");
			return;
		}
	}
	

	
	if(acquisition==1)
	{
		SetFrameTransferMode(0);  //turn off frame transfer mode
		errorValue =SetAcquisitionMode(acquisition);
    	if(errorValue!=DRV_SUCCESS)
		{
			MessagePopup("Error","Set Acquisition Mode Error");
			errorFlag=TRUE;
    	}
	}
	

	else if (acquisition==6)	   //frame transfer mode
	{
		SetFrameTransferMode(1);
		errorValue=SetNumberKinetics(number);
		if(errorValue!=DRV_SUCCESS)
			MessagePopup("Error","Set number Frames error");
	}

	else if (acquisition==5)
	{
		//nFrame=1;
		SetFrameTransferMode(0);
		SetAcquisitionMode(acquisition);
		SetKineticCycleTime(0);
	}
	
	else if (acquisition==8)
	{
		SetFrameTransferMode(0);  //turn off frame transfer mode
		errorValue =SetAcquisitionMode(1);
    	if(errorValue!=DRV_SUCCESS)
		{
			MessagePopup("Error","Set Acquisition Mode Error");
			errorFlag=TRUE;
    	}
	//	SetFrameTransferMode(1);
	//	errorValue=SetNumberKinetics(1);
	//	if(errorValue!=DRV_SUCCESS)
	//		MessagePopup("Error","Set number Frames error");		
	}		
	
	
//	else if (acquisition==0)
//	{
//		SetFrameTransferMode(0);
//		SetAcquisitionMode(acquisition);
//		SetKineticCycleTime(0);
//	}

	// set to take image, track, line, etc :     Ű κ
    errorValue=SetReadMode(readmode);
    if(errorValue!=DRV_SUCCESS)
	{
		MessagePopup("Error","Set Read Mode Error");
		errorFlag=TRUE;
    }
	//

	//CCD ӵ õ κ
	// κ      ִ  Ѵ.
	errorValue = SetVSSpeed(g_nVSNumber);
	errorValue = SetADChannel(0); 	//0,1 for 897,   0 for 885
	errorValue = SetHSSpeed(0, g_nHSNumber);
	

	////////////////////
	//shutter setting
	////////////////////
	ttl=1;			// 0:output TTL low signal to open shutter
	openclose=1;	// millisecond
	errorValue=SetShutter(ttl, shutter, openclose, openclose);
	
	
	SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor, "\nComplete Shutter Setting\n");
	if(errorValue!=DRV_SUCCESS)
		MessagePopup("Error","Shutter Error");
	
	//errorValue=SetFanMode(0);
	errorValue=SetFanMode(fan);
	if(errorValue!=DRV_SUCCESS)
		MessagePopup("Error","Fan Error");

 
	errorValue=SetPreAmpGain(2);			//CCD Dynamin Range   ǥϱ  ʿ κ
											// κ õ    ũⰡ ۾.
	SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,"Set PreAmpGain is 2-4.4X\n");
	if(errorValue!=DRV_SUCCESS)
		MessagePopup("Error", "Set PreAmpGain Error");

	
	// Set EMCCD Gain
	errorValue = SetEMCCDGain(gain);
	if (errorValue != DRV_SUCCESS)
	{
		MessagePopup("Error","EMCCD Gain error");
	}
	Sleep(250);
 
	
	// Set to image size and binning on CCD chip
	// 
	//Binning parameter
	//hBin, vBin : ׺(, )
	//hStart, hEnd : CCD chipset  / ġ
	//vStart, vEnd : CCD chipset  / ġ	

	
	if(camera==nCameraHandle1)			
	{
		if(g_nXPixels1>1000)		//DV885
		{
			errorValue=SetImage(HBin*2,VBin*2,hStart,hEnd,vStart,vEnd);
//			errorValue=SetImage(HBin*1,VBin*1,hStart,hEnd,vStart,vEnd);			
		}
		else						//DV997
		{
			errorValue=SetImage(HBin,VBin,hStart,hEnd,vStart,vEnd);    
		}
	}

	errorValue=SetTriggerMode(trigger);
	if(errorValue!=DRV_SUCCESS)
		MessagePopup("Error","Set Trigger Mode Error");		
	
	//set to acqual parameters such as exposure time, accumulation time and kinetic time
	sprintf(kkk,"\nsetting exposure time is %f\n",exptime);
	SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,kkk);

 
	// Apply actual Exposure Time
	errorValue = SetExposureTime(exptime);
	if (errorValue != DRV_SUCCESS)
	{
		MessagePopup("Error","Exposure time error");
	}
	

	//Bias Level õ κ
	// κ õ  Ư(e.x 800α)  
	//׺   0  ߶  Ѵ.
	//ð ۴ ϴ   ؼ   ϴ  .
//	errorValue=SetBaselineClamp(1);
//	if(errorValue == DRV_SUCCESS)
//	{
//			SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,"BaseLine Clamp Setting enalbe\n");
//			errorValue=SetBaselineOffset(1000);
//			if(errorValue != DRV_SUCCESS)
//				SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,"Not Set Baseline Clamp\n"); 
//	}	
	
//	if(baseline == 0)
//	{
//		errorValue=SetBaselineClamp(0);
//		SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,"SetBaseline Clamp Off\n"); 
//	}
//	if(baseline == 1)
//	{
		errorValue=SetBaselineClamp(1);
		if(errorValue == DRV_SUCCESS)
		{
			SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,"BaseLine Clamp On\n");
			errorValue=SetBaselineOffset(1000);
			if(errorValue != DRV_SUCCESS)
				SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,"Not Set Baseline Clamp\n"); 
		}			
//	}
	


	/*
	This function will return the current "Valid" acquisition timing information.
	This function shoudl be used after all the acquisitions settings have been set, 
	e.g. SetExposureTime, SetKineticCycleTime, SetReadMode etc.
	The Values returned are the actual times used in subsequent acquisitions.
	
	It is necessary to get the actual times as the system will calculate the nearest possible time. 
	E.g, if you set exposure time to be 0, the system will use the closest value(around 0.00002s)
	*/
	
	GetAcquisitionTimings(&exptime, &fAccumTime, &cycletime);  
	sprintf(kkk,"\nactual exposure time is %f,\nvalid kinetic cycle time is %f\n",exptime, cycletime);
	SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor, kkk);
	
	
	//SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,"Setting Complete\n");
	
//	errorValue=StartAcquisition();	
//	if(errorValue!=DRV_SUCCESS)
//	{
//		MessagePopup("Error","Start acquisition error");
//		AbortAcquisition();
//		return;
//	}	
}
	




////////////
void TakeBiasDark(int gain, int HBin, int VBin, float exptime, long camera,char* filepath)
{
	int 	nStatus;
	char    kkk[256];
	float 	fAccumTime,cycletime;

	int ttl, openclose;   //associate with shutter setting
	int hStart, vStart, hEnd, vEnd;
	
	int framesize;
	int nBytePerPixel 	= 2;     //because the dynamic range of CCD Camera is 14bits (need to 16bit : 2byte)
	int nByteSize;
	int nPixelSize;
	
	int	g_nVSNumber;
	int g_nHSNumber = 0;
	int number=100;

	int acquisition=6;
	int readmode=4;//image mode
	int shutter=2;//close
	int fan=0;
	//int baseline=1;
	int trigger=0;	
	int i,j,nStartPoint;
	//char filepath_temp[100];
	
	WORD* BiasDarkTemp1;
	WORD* BiasDarkTemp2;
	
	hStart=1;							//start horizontal pixel position on CCD chip
	vStart=1;							//start vertical pixel position on CCD chip
	 if(camera==nCameraHandle2) 
	{
		MessagePopup("error", "TakeBiasDark Function called with camerahandle2");
		return;
	}


		if(g_nXPixels1>1000)  //dv885
		{
			g_nVSNumber = 3;		//0; //the value determinid is 3, in case of dv885	
			hEnd=g_nXPixels1;
			vEnd=g_nYPixels1-2;	
			nhDim=((hEnd-hStart+1)/HBin)/2;			//actual horizontal size on CCD chip
			nvDim=((vEnd-vStart+1)/VBin)/2;			//actual vertical size on CCD chip			
			nhDim1=nhDim;
			nvDim1=nvDim;
		}
		else				 //dv887
		{
			g_nVSNumber = 4;		//0; //the value determinid is 4, in case of dv887
			hEnd=g_nXPixels1;
			vEnd=g_nYPixels1;
			nhDim=(hEnd-hStart+1)/HBin;			//actual horizontal size on CCD chip
			nvDim=(vEnd-vStart+1)/VBin;			//actual vertical size on CCD chip			
			nhDim1=nhDim;
			nvDim1=nvDim;
		}
		framesize=nhDim1*nvDim1;
	
	
	
	nPixelSize=framesize;
	nByteSize	=	nPixelSize * nBytePerPixel;
	
		BiasDark1 = (WORD*)malloc(nByteSize);	  //allocate the memory to store image
		BiasDarkTemp1 = (WORD*)malloc(nByteSize*number);	
	
	
	
	
    errorValue =SetAcquisitionMode(acquisition);
	
	if (acquisition==6)
	{
		SetFrameTransferMode(1);
		errorValue=SetNumberKinetics(number);
	}



	// set to take image, track, line, etc :     Ű κ
    errorValue=SetReadMode(readmode);

	errorValue = SetVSSpeed(g_nVSNumber);
	errorValue = SetADChannel(0); 	//avaliable value is 2, then 0,1
	errorValue = SetHSSpeed(0, g_nHSNumber);
	

	////////////////////
	//shutter setting
	////////////////////
	ttl=1;			// 0:output TTL low signal to open shutter
	openclose=1;	// millisecond
	errorValue=SetShutter(ttl, shutter, openclose, openclose);
	 
	errorValue=SetFanMode(fan);
	errorValue=SetPreAmpGain(2);			//CCD Dynamin Range   ǥϱ  ʿ κ
											// κ õ    ũⰡ ۾.

	// Set EMCCD Gain
	errorValue = SetEMCCDGain(gain);
	Sleep(250);
 
	

		if(g_nXPixels1>1000)		//DV885
		{
			errorValue=SetImage(HBin*2,VBin*2,hStart,hEnd,vStart,vEnd);
		}
		else						//DV997
		{
			errorValue=SetImage(HBin,VBin,hStart,hEnd,vStart,vEnd);    
		}
	


	
	errorValue=SetTriggerMode(trigger);
	
	//set to acqual parameters such as exposure time, accumulation time and kinetic time
	sprintf(kkk,"\nsetting exposure time is %f\n",exptime);
	SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,kkk);

 
	// Apply actual Exposure Time
	errorValue = SetExposureTime(exptime);
	
	errorValue=SetBaselineClamp(1);
	errorValue=SetBaselineOffset(1000);
	if(errorValue != DRV_SUCCESS)
		SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,"Not Set Baseline Clamp\n"); 
	
	GetAcquisitionTimings(&exptime, &fAccumTime, &cycletime);  

	errorValue=StartAcquisition();	
	if(errorValue!=DRV_SUCCESS)
	{
		MessagePopup("Error","Start acquisition error");
		AbortAcquisition();
		return;
	}	
	
	
	do
	{
		errorValue = GetStatus(&nStatus);
	  	if(errorValue!=DRV_SUCCESS)
		{
			MessagePopup("Status","System not initialized");
			return;
		}							  
	}  while (nStatus != DRV_IDLE);
	

		errorValue = GetAcquiredData16(BiasDarkTemp1, nhDim1*nvDim1*number);
		memset(BiasDark1,0,nhDim1*nvDim1*sizeof(WORD));   
		for(i=0;i<number;i++)
		{
			nStartPoint=nhDim1*nvDim1*i; 
			for(j=0;j<nhDim1*nvDim1;j++)
				BiasDark1[j]=BiasDark1[j]+BiasDarkTemp1[j+nStartPoint];
		}
		
		for(i=0;i<nhDim1*nvDim1;i++)
			BiasDark1[i]=BiasDark1[i]/number;
	
	DataSource=1;			// ǥϱ  
	
	//strcat(filepath_temp,filepath);
	//printf("filepath is %s\n",filepath);
		SaveBiasDark(nhDim1,nvDim1,1,exptime,nCameraHandle1,filepath);


		if(BiasDarkTemp1)
		{
			free(BiasDarkTemp1);
			BiasDarkTemp1=NULL;
		}

	

}

////////////
int SaveBiasDark(int nWidth, int nHeight, int number, float exptime, long camera, char* filepath)
{
	//-------------------------
	LARGE_INTEGER gliFreq;//, gliStart, gliEnd;
	double		diff_time;
	char 		temp[250];
	//-------------------------

	struct tm *systime;		   //need to create filename as time information
	time_t t;
	char pFileName[50];
	char temp1[50],temp2[50];
	
	
	//additional parameters
	int	current_ccd_temp;    
	char ccd_modelname[32];
	
	//using nasa fits library
	fitsfile 	*fptr;
	int 		status=0;
	long 		naxis[2], fpixel[2], nelements;
	char		filename[512];
	

	naxis[0]=nWidth;
	naxis[1]=nHeight;
	fpixel[0]=fpixel[1]=1;
	nelements=naxis[0]*naxis[1];
	
	memset(filename,NULL,512*sizeof(char));
	
	strcat(filename, filepath);
	strcat(filename,"\\");

	
	//-------------------------
	//determine during process
	if(!QueryPerformanceFrequency(&gliFreq))
	MessagePopup("Error","Cannot support the timer of high resolution\n");
	
	//QueryPerformanceCounter(&gliStart);
	marktime_start(temp1);
	
	//-------------------------

	//get systime to write file
 	t=time(NULL);
 	systime=gmtime(&t);

	if(camera == nCameraHandle2)MessagePopup("error", "You don't call 2nd camera savebiasdark function,");
	
	sprintf(pFileName,"FISS_%.2d%.2d%.2d_%.2d%.2d%.2d_A_BiasDark.fts",(*systime).tm_year+1900,
			(*systime).tm_mon+1,(*systime).tm_mday,(*systime).tm_hour,(*systime).tm_min,(*systime).tm_sec);
	
	strcat(filename, pFileName);
	
	
	//printf("filename is %s\n",filename);
	
//	fits_create_file(&fptr, pFileName, &status);
	fits_create_file(&fptr, filename, &status);
	
	fits_create_img(fptr, SHORT_IMG,2,naxis, &status);
	fits_write_date(fptr,&status);
	fits_write_key(fptr,TFLOAT,"EXPTIME",&exptime,"Second",&status);
	fits_write_key(fptr,TUINT,"HBINNING",&ccd_HBin,"",&status);
	fits_write_key(fptr,TUINT,"VBINNING",&ccd_VBin,"",&status);
	fits_write_key(fptr,TUINT,"GAIN",    &ccd_Gain,"Value Range: 0-255",&status);
	

		SetCurrentCamera(nCameraHandle1);
		GetTemperature(&current_ccd_temp);
		fits_write_key(fptr,TINT,"CCDTEMP",&current_ccd_temp,"Cooling Temperature of CCD",&status);
	
		GetHeadModel(ccd_modelname);   
		fits_write_key(fptr,TSTRING,"CCDNAME",ccd_modelname,"Productname of CCD",&status);
		fits_write_pix(fptr,TSHORT,fpixel,nelements,BiasDark1,&status); 

	
	
  
	fits_delete_key(fptr,"COMMENT",&status);
	fits_delete_key(fptr,"COMMENT",&status);
	

	fits_close_file(fptr,&status);
	

	//-------------------------	
	marktime_end(temp2);
	diff_time=time_difference();
	//QueryPerformanceCounter(&gliEnd);
	//diff_time=(double)(((double)gliEnd.QuadPart-(double)gliStart.QuadPart)/(double)(gliFreq.QuadPart));
	//diff_time=(double)((double)difftime/(float)(gliFreq.QuadPart));
	sprintf(temp,"%.3f Sec\n",diff_time);
	SetCtrlVal(mainPanel,PANEL_TB_Text_Monitor,temp); 
	//-------------------------	
		
	
	return 1;
}




void TakeImage(int nWidth, int nHeight, long camera, int i)
{   
	int nStatus;

	//***************************************
	//start to take picture
	//***************************************
	errorValue=StartAcquisition();	
	if(errorValue!=DRV_SUCCESS)
	{
		MessagePopup("Error","Start acquisition error");
		AbortAcquisition();
		return;
	}
	
	
	//      ʿ ð.
	do
	{
		errorValue = GetStatus(&nStatus);
	  	if(errorValue!=DRV_SUCCESS)
		{
			MessagePopup("Status","System not initialized");
			return;
		}							  
	}
	while (nStatus != DRV_IDLE);
	DataDump(nWidth,nHeight, camera,i);
	
	//CCD  ͸ ǻ ޸𸮷 ű Լ
}	



//Dump the data from camera to memory we already allocated
void DataDump(int nWidth, int nHeight, long camera, int i)
{
		int nPixelSize		= nWidth * nHeight;
		
		if(camera == nCameraHandle1)
		{
			errorValue = GetAcquiredData16(g_pRawData+nPixelSize*i, nPixelSize);
		}
		
		DataSource=1;			// ǥϱ  Ǵϴ .
								//  1 ̶ ȭ鿡 ǥϴ  
								//CCD   ǥѴٴ ǹ̰ ȴ.
}



void TakeImage_FTmode(int nWidth, int nHeight, int number, long camera)
{   
	int nStatus;
	int nPixelSize= nWidth * nHeight * number;
	
	errorValue=StartAcquisition();	
	if(errorValue!=DRV_SUCCESS)
	{
		MessagePopup("Error","Start acquisition error");
		AbortAcquisition();
		return;
	}

	do
	{
		errorValue = GetStatus(&nStatus);
	  	if(errorValue!=DRV_SUCCESS)
		{
			MessagePopup("Status","System not initialized");
			return;
		}							  
	} while (nStatus != DRV_IDLE);
	
	
	if(camera == nCameraHandle1)
		errorValue = GetAcquiredData16(g_pRawData, nPixelSize);


	DataSource=1;			// ǥϱ  Ǵϴ .
							//  1 ̶ ȭ鿡 ǥϴ  
							//CCD   ǥѴٴ ǹ̰ ȴ.	
}	





//upside down the raw image;		//ccd    ȭ鿡 Ѹ Լ
void   DisplayCCDImage(int nWidth, int nHeight, int number, long camera)
{
	int 			nBytePerPixel 	= 2;   
	int				nByteSize 		= nWidth * nHeight * nBytePerPixel * number;  	
	int 			i, j, k, m, nindex1, nindex2, start;

	unsigned short 	nMinValue;//= USHRT_MAX;
	unsigned short 	nMaxValue;//= 0;
	int 			nIndex;
	int				k1,k2;

	SetCtrlAttribute(mainPanel, PANEL_BYTSCL_MIN, ATTR_DIMMED, 0);
	SetCtrlAttribute(mainPanel, PANEL_BYTSCL_MAX, ATTR_DIMMED, 0);
	SetCtrlAttribute(mainPanel, PANEL_GRAPH, ATTR_DIMMED, 0);
	SetCtrlAttribute(mainPanel, PANEL_xpos, ATTR_DIMMED, 0);
	SetCtrlAttribute(mainPanel, PANEL_ypos, ATTR_DIMMED, 0);
	SetCtrlAttribute(mainPanel, PANEL_value, ATTR_DIMMED, 0);
	//SetCtrlAttribute(mainPanel, PANEL_Show_Hist, ATTR_DIMMED, 0);		

	
	if(number==1)		//just 1 frame
	{
		if(camera == nCameraHandle1)
		{
			planeDataCCD = (WORD*)g_pRawData;
			//for(i=0;i<nWidth*nHeight;i++)
			//	planeDataCCD[i]=planeDataCCD[i]-BiasDark1[i];				
			
			mean=DataMean(nhDim1,nvDim1, planeDataCCD);
			stdev=DataStdev(nhDim1, nvDim1, planeDataCCD, mean);			
		}

	}
	
	
	if(number!=1)		//take several images such as synchronization between CCD and Scanner
	{
		SetCtrlAttribute(mainPanel, PANEL_set_Select_Frame, ATTR_DIMMED, 0); //Activate Select Axis 
		SetCtrlAttribute(mainPanel, PANEL_set_Select_Frame, ATTR_MIN_VALUE, 0);
		SetCtrlAttribute(mainPanel, PANEL_set_Select_Frame, ATTR_MAX_VALUE, number-1);		
		SetCtrlAttribute(mainPanel, PANEL_SelectAxis, ATTR_DIMMED, 0); //Activate Select Axis 

		start=0;		//display only first spectrogram image from 3D data
		planeDataCCD=(WORD*)malloc(nWidth*nHeight*nBytePerPixel);
		
		if(camera == nCameraHandle1)
		{
			memcpy(planeDataCCD, g_pRawData+start, nWidth*nHeight*nBytePerPixel);  			
			for(i=0;i<nWidth*nHeight;i++)
				planeDataCCD[i]=planeDataCCD[i];//-BiasDark1[i];		
			
			mean=DataMean(nhDim1,nvDim1, planeDataCCD);
			stdev=DataStdev(nhDim1, nvDim1, planeDataCCD, mean);
		}
	
	}
	

	GetMinMax(planeDataCCD, nWidth*nHeight, &nMinValue, &nMaxValue);
	
	
	if(nMinValue>(int)(mean-3*stdev))
			k1=nMinValue;
	else
			k1=(int)(mean-3*stdev);
	if(nMaxValue<(int)(mean+3*stdev))
			k2=nMaxValue;
	else
			k2=(int)(mean+3*stdev);						


	ByteDisplay(nWidth, nHeight, planeDataCCD, k1, k2);

	SetCtrlAttribute(mainPanel, PANEL_BYTSCL_MIN, ATTR_MIN_VALUE, nMinValue);
	SetCtrlAttribute(mainPanel, PANEL_BYTSCL_MIN, ATTR_MAX_VALUE, nMaxValue);
	SetCtrlVal(mainPanel, PANEL_BYTSCL_MIN, k1);
	SetCtrlAttribute(mainPanel, PANEL_BYTSCL_MAX, ATTR_MIN_VALUE, nMinValue);
	SetCtrlAttribute(mainPanel, PANEL_BYTSCL_MAX, ATTR_MAX_VALUE, nMaxValue);
	SetCtrlVal(mainPanel, PANEL_BYTSCL_MAX, k2);						
}





	
//shut down system	
void CloseCamera(long camera)
{
	int nStatus;
	int i;
	char temp[255];

	do
	{
		errorValue = GetStatus(&nStatus);
	  	if(errorValue!=DRV_SUCCESS)
		{
			MessagePopup("Status","System not initialized1");
			return;
			//break;
		}							  
	}
	while (nStatus != DRV_IDLE);


	if(errorValue!=DRV_SUCCESS)
		MessagePopup("Error","Error switching cooler off");
	

	////////////////////
	//shutter setting
	////////////////////
	
	errorValue=SetShutter(1, 0, 1, 1);
	SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,"Set to Shutter Auto Mode\n");
	if(errorValue!=DRV_SUCCESS)
		MessagePopup("Error","Shutter Error");
	
	
	errorValue=SetFanMode(2);	//fan off
	
 
	

		
	
	

	errorValue=ShutDown();
	if(errorValue!=DRV_SUCCESS)
		MessagePopup("Error","Error shutting down");
	
	
	if(camera==nCameraHandle1)
	{
		
		if (g_pRawData)				 //g_pRawData  Ǹ װ ߴ planeDataCCD  .
		{							 //  planeDataCCD  ʿ䰡 .!!
			free(g_pRawData);
			g_pRawData = NULL;
		}
	}
	


	if(pusFitsData)
	{
		free(pusFitsData);
		pusFitsData=NULL;
	}
	
	if(pFitsData)
	{
		free(pFitsData);
		pFitsData=NULL;
	}
	
	if(planeDataCCD)
	{
		free(planeDataCCD);
		planeDataCCD=NULL;
	}
	
	if(BiasDark1)
	{
		free(BiasDark1);
		BiasDark1=NULL;
	}
	

	SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor,"Complete Shut Down\n");
	
	CCD_status=0;
}





//save the data as bitmap files
int SaveBitmap(int nWidth, int nHeight, void* pData, char* pFileName, int nMinValue, int nMaxValue)
{
	int i;  
	int nBmpWidth;
	FILE* pFile;
	long spanvalue;
	
	float temp;
	unsigned short* realData=(unsigned short*)pData;
	
	BITMAPFILEHEADER bmfh;
	BITMAPINFOHEADER bmih;
	RGBQUAD* pRGB = NULL;	  	
	

	BYTE* pDisplayData=(BYTE*)malloc(nWidth*nHeight*sizeof(BYTE));
	
	spanvalue=(long)nMaxValue-(long)nMinValue;

	bmfh.bfType=0x4d42;
	bmfh.bfReserved1=0;
	bmfh.bfReserved2=0;
	bmfh.bfOffBits=0;

	bmih.biSize		= sizeof(BITMAPINFOHEADER);
	bmih.biWidth	= nWidth;
	bmih.biHeight	= nHeight;
	bmih.biPlanes	= 1;
	bmih.biBitCount	= 8;

	nBmpWidth= (bmih.biBitCount * bmih.biWidth + 31) / 32 * 4;

	bmih.biCompression		= BI_RGB;
	bmih.biSizeImage		= bmih.biBitCount/8 * nBmpWidth * nHeight;
	bmih.biXPelsPerMeter	= 0;
	bmih.biYPelsPerMeter	= 0;
	bmih.biClrUsed			= 256;
	bmih.biClrImportant		= 256;

	bmfh.bfSize = sizeof(BITMAPFILEHEADER)
					+ sizeof(BITMAPINFOHEADER)
					+ sizeof(RGBQUAD) * bmih.biClrUsed
					+ bmih.biSizeImage;
	bmfh.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * bmih.biClrUsed;
	pRGB =  (RGBQUAD*)malloc(bmih.biClrUsed * sizeof(RGBQUAD));

	
	for (i=0;i<bmih.biClrUsed;i++)
	{
		pRGB[i].rgbRed  = i;
		pRGB[i].rgbGreen= i;
		pRGB[i].rgbBlue = i;
		pRGB[i].rgbReserved =0;
	}

   
	for(i=0;i<nWidth*nHeight;i++)
	{
		temp=(((float)realData[i]-(float)nMinValue)/(float)spanvalue);
		if(temp>1) temp=1;
		if(temp<0) temp=0;
		pDisplayData[i]=(int)(temp*(float)255.0);
	}

	pFile = fopen(pFileName,"wb");
	fwrite(&bmfh,sizeof(BITMAPFILEHEADER), 1, pFile);
	fwrite(&bmih,sizeof(BITMAPINFOHEADER), 1, pFile);
	fwrite(pRGB, sizeof(RGBQUAD),256,pFile);
	fwrite(pDisplayData, sizeof(BYTE), bmih.biSizeImage, pFile);
	fclose(pFile);
	
	free(pRGB);
	free(pDisplayData);
	return 1;
}
//



int SaveFits(int nWidth, int nHeight, int number, float exptime, long camera, char* time1, char* time2, float elapsetime, char* filepath)
{
	//-------------------------
	LARGE_INTEGER gliFreq;//, gliStart, gliEnd;
	double		diff_time;
	char 		temp[250];
	//-------------------------

	struct tm *systime;		   //need to create filename as time information
	time_t t;
	char pFileName[50];
	char temp1[50],temp2[50];
	
	
	//additional parameters
	int	current_ccd_temp;    
	char ccd_modelname[32];

	//using nasa fits library
	fitsfile 	*fptr;
	int 		status=0;
	long 		naxis[3], fpixel[3], nelements, naxis_one[2],fpixel_one[2];
	int			TabPanelHandle;
	char		fits_contents[250];
	int			real_scan_size=scan_size*2;
	int			value;
	char		filename[512];   

	

	
	//scanner parameter
	//extern int Sacnner_status;
	//extern int scan_mode;
	
	GetPanelHandleFromTabPage(FISSpanelSetting, SetFISS_setTAB, 3, &TabPanelHandle);
//	GetCtrlVal(TabPanelHandle, setMISC_MISC_Observer, &fits_contents);	
	
	memset(filename,NULL,512*sizeof(char));
	strcat(filename, filepath);
	strcat(filename,"\\");

	if(number==1)
	{
		naxis_one[0]=nWidth;
		naxis_one[1]=nHeight;
		fpixel_one[0]=fpixel_one[1]=1;
		nelements=naxis_one[0]*naxis_one[1];
	}
		
	if(number>1)
	{
		naxis[0]=nWidth;
		naxis[1]=nHeight;
		naxis[2]=number;
		fpixel[0]=fpixel[1]=fpixel[2]=1;
		nelements=naxis[0]*naxis[1]*naxis[2];		
	}
	


	
	//-------------------------
	//determine during process
	if(!QueryPerformanceFrequency(&gliFreq))
	MessagePopup("Error","Cannot support the timer of high resolution\n");
	
	//QueryPerformanceCounter(&gliStart);
	marktime_start(temp1);
	
	//-------------------------

	//get systime to write file
 	t=time(NULL);
 	systime=gmtime(&t);

	if(camera == nCameraHandle1)   
		sprintf(pFileName,"FISS_%.2d%.2d%.2d_%.2d%.2d%.2d_A.fts",(*systime).tm_year+1900,
			(*systime).tm_mon+1,(*systime).tm_mday,(*systime).tm_hour,(*systime).tm_min,(*systime).tm_sec);
	if(camera == nCameraHandle2)
		sprintf(pFileName,"FISS_%.2d%.2d%.2d_%.2d%.2d%.2d_B.fts",(*systime).tm_year+1900,
			(*systime).tm_mon+1,(*systime).tm_mday,(*systime).tm_hour,(*systime).tm_min,(*systime).tm_sec);
	
	strcat(filename,pFileName);
  
//	fits_create_file(&fptr, pFileName, &status);
	fits_create_file(&fptr, filename, &status);  
	
	if(number>1)
	{
		fits_create_img(fptr, SHORT_IMG,3,naxis, &status);
		fits_write_date(fptr,&status);
	}
	if(number==1)
	{
		fits_create_img(fptr, SHORT_IMG,2,naxis_one, &status);

	}
	fits_write_date(fptr,&status);		 
	fits_write_key(fptr,TFLOAT,"EXPTIME",&exptime,"Second",&status);
	fits_write_key(fptr,TUINT,"HBINNING",&ccd_HBin,"",&status);
	fits_write_key(fptr,TUINT,"VBINNING",&ccd_VBin,"",&status);
	fits_write_key(fptr,TUINT,"GAIN",    &ccd_Gain,"Value Range: 0-255",&status);
	fits_write_key(fptr,TSTRING,"STRTIME",time1,"Scan Start Time",&status);
	fits_write_key(fptr,TSTRING,"ENDTIME",time2,"Scan Finish Time", &status);
	fits_write_key(fptr,TFLOAT,"ELAPTIME",&elapsetime,"Elapse Time During Scanning(Second)",&status);
	
	if(Scanner_status==1) //scanner turn on
	{
		switch(scan_mode)
		{
			case 0:		//spectrograph mode
				fits_write_key_str(fptr,"FISSMODE","Spectrograph","FISS Mode",&status);
				fits_write_key(fptr,TFLOAT,"STEPTIME",&scan_duration,"1 Step Duration time(millisecond)",&status);
				fits_write_key(fptr,TFLOAT,"STEPSIZE",&real_scan_size,"1 step size to move scanner(micrometer)",&status);
				break;
			case 1:		//imaging mode
				fits_write_key_str(fptr,"FISSMODE","Imaging","FISS Mode",&status);
				fits_write_key(fptr,TFLOAT,"SCNSPEED",&scan_speed,"Scan Speed(micrometer/second)",&status);
				fits_write_key(fptr,TFLOAT,"SCANDIST",&scan_stroke,"Distance to Scan(millimeter)",&status);
				break;
			case -1:	//None
				fits_write_key_str(fptr,"FISSMODE","None(Only CCD)","FISS Mode",&status);
				break;
		}
	}
	
	if(Scanner_status==0) //scanner turn off
	{
		fits_write_key_str(fptr,"FISSMODE","None(Only CCD)","FISS Mode",&status);
	}
		
		
	
	
	if(camera==nCameraHandle1)
	{
		SetCurrentCamera(nCameraHandle1);
		GetTemperature(&current_ccd_temp);
		fits_write_key(fptr,TINT,"CCDTEMP",&current_ccd_temp,"Cooling Temperature of CCD",&status);
	
		GetHeadModel(ccd_modelname);   
		fits_write_key(fptr,TSTRING,"CCDNAME",ccd_modelname,"Productname of CCD",&status);
		fits_write_key_str(fptr,"WAVELEN","6562.8","CCD1 Wavenength(angstrom)",&status);
		fits_write_key(fptr,TUINT,"CCD_XPIX",&g_nXPixels1,"Full Pixel Size of X",&status);
		fits_write_key(fptr,TUINT,"CCD_YPIX",&g_nYPixels1,"Full Pixel Size of Y",&status);
		
		if(number==1)
			fits_write_pix(fptr,TSHORT,fpixel_one,nelements,g_pRawData,&status);
		if(number>1)
			fits_write_pix(fptr,TSHORT,fpixel,nelements,g_pRawData,&status); 
	}
	

	
	fits_delete_key(fptr,"COMMENT",&status);
	fits_delete_key(fptr,"COMMENT",&status);
	fits_delete_key(fptr,"EXTEND",&status);
	
	    
	fits_write_key(fptr, TSTRING, "OBSERVER", m_observername, "The Name of Main Observer",&status);
	
	
	
	switch(m_ringvalue)
	{
		case -1:	//none
				fits_write_key(fptr, TSTRING, "TARGET", "NONE", "NONE", &status);
				break;
		case 0: 	//near disk center
				fits_write_key(fptr, TSTRING, "TARGET", "Near Disk Center", "Observation Target", &status);  
				break;
		case 1:     //close to or on the limb
				fits_write_key(fptr, TSTRING, "TARGET", "Close to or on the Limb", "Observation Target", &status); 
				break;
		case 2:    //quiet region
				fits_write_key(fptr, TSTRING, "TARGET", "Quiet Region", "Observation Target", &status); 
				break;
		case 3:    //active region
				fits_write_key(fptr, TSTRING, "TARGET", "Active Region", "Observation Target", &status); 
				
				fits_write_key(fptr, TSTRING, "ARNUMBER",m_arnumber, "Active Region Number",&status);				
				break;	
		case 4:	   //corona hole
				fits_write_key(fptr, TSTRING, "TARGET", "Corona Hole", "Observation Target", &status); 
				break;
		case 5:	   //sunspot
				fits_write_key(fptr, TSTRING, "TARGET", "Sunspot", "Observation Target", &status); 
				break;
		case 6:	   //filament
				fits_write_key(fptr, TSTRING, "TARGET", "Filament", "Observation Target", &status); 
				break;
		case 7:	   //prominence
				fits_write_key(fptr, TSTRING, "TARGET", "Prominence", "Observation Target", &status); 
				break;		
		case 8:     //user defined object
				GetCtrlVal(TabPanelHandle, setMISC_MISC_Target, fits_contents);
				fits_write_key(fptr, TSTRING, "TARGET", fits_contents, "Observation Target", &status);			
				break;
	}	
	
	
	fits_write_key(fptr, TINT, "TEL_XPOS", &m_solarxpos, "X position of Telescope on the Sun", &status);
	fits_write_key(fptr, TINT, "TEL_YPOS" ,&m_solarypos, "Y position of Telescope on the Sun", &status);     

	fits_close_file(fptr,&status);
	

	//-------------------------	
	marktime_end(temp2);
	diff_time=time_difference();
//	QueryPerformanceCounter(&gliEnd);
//	diff_time=(double)(((double)gliEnd.QuadPart-(double)gliStart.QuadPart)/(double)(gliFreq.QuadPart));
	sprintf(temp,"%.3f Sec\n",diff_time);
	SetCtrlVal(mainPanel,PANEL_TB_Text_Monitor,temp); 
	//-------------------------	
	  
	return 1;
}



int SaveFlatFits(int nWidth, int nHeight, int number, float exptime, long camera, char* time1, char* time2, float elapsetime, char* filepath)
{
	//-------------------------
	double		diff_time;
	char 		temp[250];
	//-------------------------

	LARGE_INTEGER gliFreq;
	struct tm *systime;		   //need to create filename as time information
	time_t t;
	char pFileName[250];
	char temp1[150],temp2[150];
	char filename[512];
	
	//additional parameters
	int	current_ccd_temp;    
	char ccd_modelname[32];

	//using nasa fits library
	fitsfile 	*fptr;
	int 		status=0;
	long 		nelements, naxis[3],fpixel[3];
	long		naxis_one[2],fpixel_one[2],nelements_one; //for saving 2d images
	int 		PixelByte=32;
	//int
	
	memset(filename,NULL,512*sizeof(char));

	naxis[0]=nWidth;
	naxis[1]=nHeight;
	naxis[2]=number;
	fpixel[0]=fpixel[1]=fpixel[2]=1;
	nelements=naxis[0]*naxis[1]*naxis[2];
	
	naxis_one[0]=nWidth;
	naxis_one[1]=nHeight;
	fpixel_one[0]=fpixel_one[1]=1;
	nelements_one=naxis_one[0]*naxis_one[1];
	
	


	//-------------------------
	//determine during process
	if(!QueryPerformanceFrequency(&gliFreq))
	MessagePopup("Error","Cannot support the timer of high resolution\n");
	marktime_start(temp1);
	//-------------------------

	//get systime to write file
 	t=time(NULL);
 	systime=gmtime(&t);
	
	strcat(filename, filepath);
	strcat(filename, "\\");
	

	if(camera == nCameraHandle1) 
		sprintf(pFileName,"FISS_%.2d%.2d%.2d_%.2d%.2d%.2d_A_Flat.fts",(*systime).tm_year+1900,
			(*systime).tm_mon+1,(*systime).tm_mday,(*systime).tm_hour,(*systime).tm_min,(*systime).tm_sec);

	
	strcat(filename, pFileName);
  
//	fits_create_file(&fptr, pFileName, &status);
	fits_create_file(&fptr, filename, &status);

	if(number==1)
		fits_create_img(fptr,FLOAT_IMG,2,naxis_one, &status);
	
	if(number>1)
		fits_create_img(fptr, FLOAT_IMG,3,naxis, &status);
	
	fits_write_date(fptr,&status);		 
	fits_write_key(fptr,TFLOAT,"EXPTIME",&exptime,"Second",&status);
	fits_write_key(fptr,TUINT,"HBINNING",&ccd_HBin,"",&status);
	fits_write_key(fptr,TUINT,"VBINNING",&ccd_VBin,"",&status);
	fits_write_key(fptr,TUINT,"GAIN",    &ccd_Gain,"Value Range: 0-255",&status);
	fits_write_key(fptr,TSTRING,"STRTIME",time1,"Scan Start Time",&status);
	fits_write_key(fptr,TSTRING,"ENDTIME",time2,"Scan Finish Time", &status);
	fits_write_key(fptr,TFLOAT,"ELAPTIME",&elapsetime,"Elapse Time During Scanning(Second)",&status);
	//fits_write_key(fptr,TINT,"BITPIX",&PixelByte,"number of bits per data pixel",&status);

	if(camera==nCameraHandle1)
	{
		SetCurrentCamera(nCameraHandle1);
		GetTemperature(&current_ccd_temp);
		fits_write_key(fptr,TINT,"CCDTEMP",&current_ccd_temp,"Cooling Temperature of CCD",&status);
	
		GetHeadModel(ccd_modelname);   
		fits_write_key(fptr,TSTRING,"CCDNAME",ccd_modelname,"Productname of CCD",&status);
		fits_write_key_str(fptr,"WAVELEN","6562.8","CCD1 Wavenength(angstrom)",&status);
		fits_write_key(fptr,TUINT,"CCD_XPIX",&g_nXPixels1,"Full Pixel Size of X",&status);
		fits_write_key(fptr,TUINT,"CCD_YPIX",&g_nYPixels1,"Full Pixel Size of Y",&status);
		
		if(number==1)
			fits_write_pix(fptr,TFLOAT, fpixel_one,nelements_one,flatcube1,&status);
		
		if(number>1)
			fits_write_pix(fptr,TFLOAT,fpixel,nelements,flatcube1,&status);
/*		
		if(flatdata1)
		{
			free(flatdata1);
			flatdata1=NULL;
		}
		
		if(flatcube1)
		{
			free(flatcube1);
			flatcube1=NULL;
		}
*/
	}
	

	
	fits_delete_key(fptr,"COMMENT",&status);
	fits_delete_key(fptr,"COMMENT",&status);
	fits_delete_key(fptr,"EXTEND",&status);
	
	fits_write_comment(fptr,"BiasDark Subtracted",&status);
	sprintf(temp,"averaged image : %d at each grating angle",g_nFrameNumber1);
	fits_write_comment(fptr,temp,&status);
	
	
	//fits_write_key(fptr,TUINT,"INTG_FRM",&number,"Number of Integrated Images",&status);
	fits_close_file(fptr,&status);
	

	//-------------------------	
	marktime_end(temp2);
	diff_time=time_difference();
	sprintf(temp,"%.3f Sec\n",diff_time);
	
	SetCtrlVal(mainPanel,PANEL_TB_Text_Monitor,temp); 
	//-------------------------	
 
	return 1;
}






//޴ ư Open  BMP ȭ   Ǵ Լ
int OpenBitmap(char* filename)
{
	int i,nindex1,nindex2;
	
	FILE* fp;
	BITMAPINFOHEADER infoheader;
	BITMAPFILEHEADER fileheader;
	RGBQUAD rgb[256];
	BYTE* displayimage;
	BYTE* upsidedownimage;
	
	
	fp=fopen(filename,"rb");
	if(fp==NULL){
		MessagePopup("Error","No File Exists");
		return 0;
	}
	fread(&fileheader,sizeof(BITMAPFILEHEADER),1,fp);
	if(fileheader.bfType != 0x4d42){
		MessagePopup("Error","No BMP File");
		return 0;
	}
	fread(&infoheader,sizeof(BITMAPINFOHEADER),1,fp);
	if(infoheader.biBitCount!=8){
		MessagePopup("Error","Bad File Format!");
		return 0;
	}
	fread(rgb,sizeof(RGBQUAD),256,fp);
	
	displayimage=(BYTE*)malloc(infoheader.biSizeImage*sizeof(BYTE));
	upsidedownimage=(BYTE*)malloc(infoheader.biSizeImage*sizeof(BYTE));	   // ̸
	
	
	fread(displayimage,sizeof(BYTE), infoheader.biSizeImage,fp);
	
	//image flip process
	for(i=0;i<infoheader.biHeight;i++)
	{
		nindex2=infoheader.biWidth*(infoheader.biHeight-1-i);
		nindex1=infoheader.biWidth*i;
		memcpy(upsidedownimage+nindex2,displayimage+nindex1,infoheader.biWidth);
	}

	
	fclose(fp);

	
	//display on panel
 	if(!GraphicPanel)
		if ((GraphicPanel = LoadPanel (0, "FISS.uir", GRAPHICPNL)) < 0)
			return -1;	
	
	DisplayPanel(GraphicPanel);
	
	OpenGraphicPanel(
						infoheader.biBitCount, 
						infoheader.biWidth,
						infoheader.biHeight, 
						(int*)rgb,
						(char*)upsidedownimage
						);

	free(displayimage);
	 
	return 1;
}



int OpenFitsFile(char* filename)
{
	unsigned short 	nMinValue;
	unsigned short 	nMaxValue;
	char 			temp[10];
	int			k1,k2;
	
	/////cfitsio parameter
	fitsfile*	fptr;
	int			status=0;
	int			i;
	int			c,k,byteperpixel,datasize;
	long		fpixel[3]={1,1,1}, nelements;
	long		fpixel_2D[2]={1,1};
	///////////////////////
	
	//int			radiogroup_index=0;
	if(!GraphicPanel)if ((GraphicPanel = LoadPanel (0, "FISS.uir", GRAPHICPNL)) < 0)
		return -1;	
	
	DisplayPanel(GraphicPanel);	
		
	if(pFitsData)
	{
		free(pFitsData);
		pFitsData=NULL;
	}	
	if(pusFitsData)
	{
		free(pusFitsData);
		pusFitsData=NULL;
	}
	

	c=0;
	for(i=0;i<256*4;i+=4)
	{
		colortable[i]=c;
		colortable[i+1]=c;
		colortable[i+2]=c;
		colortable[i+3]=0;
		c++;
	}

	
	
	fits_open_file(&fptr, filename, READONLY, &status);
	fits_read_key(fptr,TINT,"BITPIX",&FitsHeader.nBitsPerPixel,NULL,&status);
	fits_read_key(fptr,TINT,"NAXIS",&FitsHeader.nAxis,NULL,&status);
	fits_read_key(fptr,TINT,"NAXIS1",&FitsHeader.nAxis1,NULL,&status);
	fits_read_key(fptr,TINT,"NAXIS2",&FitsHeader.nAxis2,NULL,&status);
	fits_read_key(fptr,TFLOAT,"EXPTIME",&FitsHeader.ExpTime,NULL,&status);  
	

	byteperpixel=FitsHeader.nBitsPerPixel/8;  	
	FitsHeader.nBytesPerPixel=FitsHeader.nBitsPerPixel/8;
	
 	if(FitsHeader.nAxis==2)
	{
//		SetCtrlAttribute(mainPanel, PANEL_set_Select_Frame, ATTR_DIMMED, 1); //deActivate Select Axis 
//		SetCtrlAttribute(mainPanel, PANEL_SelectAxis, ATTR_DIMMED, 1);		
		
		nelements=(long)FitsHeader.nAxis1*(long)FitsHeader.nAxis2;	
		datasize=FitsHeader.nAxis1*FitsHeader.nAxis2*byteperpixel;          				
	}

	if(FitsHeader.nAxis==3)
	{
		fits_read_key(fptr,TINT,"NAXIS3",&FitsHeader.nAxis3,NULL,&status);
		nelements=(long)FitsHeader.nAxis1*(long)FitsHeader.nAxis2*(long)FitsHeader.nAxis3;	
		datasize=FitsHeader.nAxis1*FitsHeader.nAxis2*FitsHeader.nAxis3*byteperpixel; 
		
		 
	}	
	
	
	pFitsData=(BYTE*)malloc(datasize);
	if(pFitsData==NULL)
	{
		printf("Cannot allocate pFitsData Memory\n");
		return 0;
	}
	
	if(FitsHeader.nAxis==2)
	{
		fits_read_pix(fptr,TSHORT, fpixel_2D, nelements, NULL, pFitsData,NULL, &status);
		fits_close_file(fptr,&status);
		
		mean=DataMean(FitsHeader.nAxis1, FitsHeader.nAxis2, pFitsData);
		stdev=DataStdev(FitsHeader.nAxis1, FitsHeader.nAxis2, pFitsData, mean);
		
		GetMinMax(pFitsData, FitsHeader.nAxis1*FitsHeader.nAxis2, &nMinValue, &nMaxValue);
		
		if(nMinValue>(int)(mean-3*stdev))
			k1=nMinValue;
		else
			k1=(int)(mean-3*stdev);
		if(nMaxValue<(int)(mean+3*stdev))
			k2=nMaxValue;
		else
			k2=(int)(mean+3*stdev);
				
		ByteDisplay(FitsHeader.nAxis1, FitsHeader.nAxis2, pFitsData, k1, k2); 
		//ByteDisplay_diff(FitsHeader.nAxis1, FitsHeader.nAxis2, pFitsData);
	}
	if(FitsHeader.nAxis==3)
	{
		SetCtrlAttribute(mainPanel, PANEL_set_Select_Frame, ATTR_DIMMED, 0); //Activate Select Axis 
		SetCtrlAttribute(mainPanel, PANEL_SelectAxis, ATTR_DIMMED, 0);
		
		k=0;
		fits_read_pix(fptr,TSHORT, fpixel, nelements, NULL, pFitsData,NULL, &status);
		fits_close_file(fptr,&status);
		pusFitsData=(unsigned short*)malloc(FitsHeader.nAxis1*FitsHeader.nAxis2*sizeof(unsigned short));
		memcpy(pusFitsData,pFitsData+FitsHeader.nAxis1*FitsHeader.nAxis2*k*byteperpixel,FitsHeader.nAxis1*FitsHeader.nAxis2*sizeof(unsigned short));
		
		mean=DataMean(FitsHeader.nAxis1, FitsHeader.nAxis2, pusFitsData);
		stdev=DataStdev(FitsHeader.nAxis1, FitsHeader.nAxis2, pusFitsData, mean);
		
		GetMinMax(pusFitsData, FitsHeader.nAxis1*FitsHeader.nAxis2,&nMinValue,&nMaxValue);    

		if(nMinValue>(int)(mean-3*stdev))
			k1=nMinValue;
		else
			k1=(int)(mean-3*stdev);
		if(nMaxValue<(int)(mean+3*stdev))
			k2=nMaxValue;
		else
			k2=(int)(mean+3*stdev);		
		
		ByteDisplay(FitsHeader.nAxis1, FitsHeader.nAxis2, pusFitsData, k1, k2);  	
		
		//3D ͸  Ŀ  ̵带 ϴ κ
		SetCtrlAttribute(mainPanel, PANEL_set_Select_Frame, ATTR_MIN_VALUE, 0);
		SetCtrlAttribute(mainPanel, PANEL_set_Select_Frame, ATTR_MAX_VALUE, FitsHeader.nAxis3-1);
		SetCtrlVal(mainPanel, PANEL_set_Select_Frame, 0);
		
		//Radio_SetMarkedOption(mainPanel, PANEL_SelectAxis, &radiogroup_inde);
	}

	DataSource=0;			   //ȭ鿡 ǥõǴ Ͱ Fits File о ǥϴ 
	
	 
	//ȭ а ȭ鿡 ǥ  ϸ ϴ ̵带 Ѵ 
	SetCtrlAttribute(mainPanel, PANEL_BYTSCL_MIN, ATTR_MIN_VALUE, nMinValue);
	SetCtrlAttribute(mainPanel, PANEL_BYTSCL_MIN, ATTR_MAX_VALUE, nMaxValue);
	SetCtrlVal(mainPanel, PANEL_BYTSCL_MIN, k1);
	SetCtrlAttribute(mainPanel, PANEL_BYTSCL_MAX, ATTR_MIN_VALUE, nMinValue);
	SetCtrlAttribute(mainPanel, PANEL_BYTSCL_MAX, ATTR_MAX_VALUE, nMaxValue);
	SetCtrlVal(mainPanel, PANEL_BYTSCL_MAX, k2);
	 
	return 1;
}



//display data on Display pannel and plot pannel
//using only naxis=2
//data is pusFitsData
void ByteDisplay(int axis1, int axis2, void* data, unsigned short minvalue, unsigned short maxvalue)
{
	unsigned short* RealValueData;
	BYTE* ScalingData;
	
	double* hist_data;
	long spanvalue;
	int nIndex1,nIndex2;

	
	int hist_array[100];
	double xarray[100];
	int i,j;
	float tempvalue;



	
	ScalingData=(BYTE*)malloc(axis1*axis2);
	hist_data=(double*)malloc(axis1*axis2*sizeof(double));
	
	RealValueData= (unsigned short*)data;
	spanvalue=(long)maxvalue-(long)minvalue;
	
	//byte scaling and upside down
	for(j=0; j<axis2; j++)
		for(i=0; i<axis1; i++)
		{
			nIndex1=axis1*j+i;
			nIndex2=axis1*(axis2-1-j)+i;
			tempvalue= (((float)RealValueData[nIndex2] - (float)minvalue) / (float)spanvalue);
			if (tempvalue>1) tempvalue=1;
			if (tempvalue<0) tempvalue=0;
			ScalingData[nIndex1] = (int) (tempvalue*(float)255.0);
			hist_data[nIndex1]=RealValueData[nIndex2];
		}
								  
	
	

	OpenGraphicPanel(8, axis1,axis2, (int*)colortable, (char*)ScalingData); 
	
	
	if(minvalue<maxvalue)
	{
		if(plotHandle)
		{
			DeleteGraphPlot(mainPanel, PANEL_GRAPH, plotHandle, VAL_IMMEDIATE_DRAW);
		}
		Histogram(hist_data, axis1*axis2, (double)minvalue, (double)maxvalue,  hist_array, xarray, 100);
		plotHandle = PlotXY(mainPanel, PANEL_GRAPH, xarray, hist_array, 100, VAL_DOUBLE, VAL_INTEGER, VAL_VERTICAL_BAR, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_BLUE);
	}

	
	free(hist_data);

	

}




void ByteDisplay_diff(int axis1, int axis2, void* data)
{
	signed short* RealValueData;
	signed short* flipData;
	BYTE* ScalingData;
	
	long spanvalue;
	int nIndex1,nIndex2;
	
	int i,j;
	float tempvalue;

	signed short minvalue,maxvalue;
	minvalue=SHRT_MAX;
	maxvalue=SHRT_MIN;
	
	

	ScalingData=(BYTE*)malloc(axis1*axis2);
	flipData=(signed short*)malloc(axis1*axis2*sizeof(signed short));

	RealValueData= (signed short*)data;


	for(j=0; j<axis2; j++)
	{
		for(i=0; i<axis1; i++)
		{
			nIndex1=axis1*j+i;
			nIndex2=axis1*(axis2-1-j)+i;
			
			flipData[nIndex1]=RealValueData[nIndex2]-RealValueData[nIndex1];
		}
	}
	
	for(i=0;i<axis1*axis2;i++)
	{
			maxvalue=(maxvalue>flipData[i])?maxvalue:flipData[i];
			minvalue=(minvalue<flipData[i])?minvalue:flipData[i];		
	}
	

	if(minvalue<0)
	{
		for(i=0;i<axis1*axis2;i++)
			flipData[i]=flipData[i]+minvalue*(-1);
	}
	maxvalue=maxvalue-minvalue;
	minvalue=0;
	
	spanvalue=(long)maxvalue-(long)minvalue;  	

	for(j=0; j<axis1*axis2; j++)
	{
			tempvalue= (((float)flipData[j] - (float)minvalue) / (float)spanvalue);
			if (tempvalue>1) tempvalue=1;
			if (tempvalue<0) tempvalue=0;
			ScalingData[j] = (int) (tempvalue*(float)255.0);
			//hist_data[nIndex1]=RealValueData[nIndex2];
	}	

	
	

	OpenGraphicPanel(8,axis1,axis2,(int*)colortable, (char*)ScalingData);
	
	
	free(ScalingData);
}





//plot   ..ȭ ǥõǴ  Ʒ  ̹Ƿ װͿ ؼ  .
//mode y-based:0, x-based:1
void Display_Line_Profile(int xposition, int yposition, int axis1, int axis2, void* data, int mode)
{
	//int plot;
	unsigned short* realdata;
	unsigned short* profile;
	int pos;
	int pos_coord;
	int j;
	unsigned short minvalue, maxvalue;
	int numberofpoints;
	char temp[10];
	
	realdata=(unsigned short*)data;
	
	if(plotHandle)
		DeleteGraphPlot(mainPanel, PANEL_GRAPH, plotHandle, VAL_IMMEDIATE_DRAW);		
	

	if(mode==0)		   //horizontal profile
	{
		numberofpoints=axis1;
		profile=(unsigned short*)malloc(axis1*2);
	
		//consider upside down
		//original code : pos=yposition*axis1
		pos=((axis2-1)-yposition)*axis1;		 //Ư   ù°  ġ
		pos_coord=pos+xposition;				//Ư ࿡ x Ǹŭ  :ġ(1D)
		memcpy(profile,realdata+ pos,axis1*2);  //Ư  ͸ Ѵ.
		
		plotHandle=PlotY(mainPanel, PANEL_GRAPH, profile, numberofpoints, VAL_UNSIGNED_SHORT_INTEGER, VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_GREEN);
	}
	
	free(profile);
	
	sprintf(temp,"%d", realdata[pos_coord]);
	SetCtrlVal(mainPanel, PANEL_value, temp);
}






void Display3DFits(int axis, int frames, unsigned short* nMinValue, unsigned short* nMaxValue)
{
	int 			i,j,k,nIndex1,nIndex2;
	int				nStartPoint;   
	char			temp[10];
	int				k1,k2;

	if(pusFitsData)
	{
		free(pusFitsData);
		pusFitsData=NULL;
	}

	if(axis == 0 )			//X-Y plane
	{
		pusFitsData = (unsigned short*)malloc(FitsHeader.nAxis1*FitsHeader.nAxis2*sizeof(unsigned short));
		k=frames;

		nStartPoint=FitsHeader.nAxis1*FitsHeader.nAxis2*k;
		memcpy(pusFitsData, pFitsData+nStartPoint*FitsHeader.nBytesPerPixel, FitsHeader.nAxis1*FitsHeader.nAxis2*FitsHeader.nBytesPerPixel);
		
		mean=DataMean(FitsHeader.nAxis1, FitsHeader.nAxis2, pusFitsData);
		stdev=DataStdev(FitsHeader.nAxis1, FitsHeader.nAxis2, pusFitsData, mean);

		GetMinMax(pusFitsData,FitsHeader.nAxis1*FitsHeader.nAxis2,&(*nMinValue),&(*nMaxValue));
		
		if(*nMinValue>(int)(mean-3*stdev))
			k1=(*nMinValue);
		else
			k1=(int)(mean-3*stdev);
		if(*nMaxValue<(int)(mean+3*stdev))
			k2=(*nMaxValue);
		else
			k2=(int)(mean+3*stdev);
		
		
		ByteDisplay(FitsHeader.nAxis1,FitsHeader.nAxis2, pusFitsData,k1,k2);		
	}   

	if(axis == 1)			 //Z-Y plane
	{
		pusFitsData =   (unsigned short*)malloc(FitsHeader.nAxis3*FitsHeader.nAxis2*sizeof(unsigned short));
		k=frames;

		for(j=0; j<FitsHeader.nAxis2; j++)
		{
			for(i=0; i<FitsHeader.nAxis3; i++)
 			{
				nIndex1=FitsHeader.nAxis1*FitsHeader.nAxis2*i+FitsHeader.nAxis1*j+k;
				nIndex2=FitsHeader.nAxis3*j+i;
				memcpy(pusFitsData+nIndex2,pFitsData+nIndex1*FitsHeader.nBytesPerPixel,sizeof(unsigned short));
			}
	 	}
		
		mean=DataMean(FitsHeader.nAxis3, FitsHeader.nAxis2, pusFitsData);
		stdev=DataStdev(FitsHeader.nAxis3, FitsHeader.nAxis2, pusFitsData, mean);	
		
		GetMinMax(pusFitsData,FitsHeader.nAxis3*FitsHeader.nAxis2,&(*nMinValue),&(*nMaxValue));  		
		
		if(*nMinValue>(int)(mean-3*stdev))
			k1=(*nMinValue);
		else
			k1=(int)(mean-3*stdev);
		if(*nMaxValue<(int)(mean+3*stdev))
			k2=(*nMaxValue);
		else
			k2=(int)(mean+3*stdev);		
 
		
		ByteDisplay(FitsHeader.nAxis3,FitsHeader.nAxis2,pusFitsData,k1,k2);   
	}	
}








//CCD    ȭ鿡 ѷִ Լ.
void Display3D_CCD(int axis, int frames, unsigned short* nMinValue, unsigned short* nMaxValue, long camera) 
{
	int				nStartPoint;
	
	//unsigned short	nMinValue, nMaxValue;
	int				i,j,k;
	int				nIndex1, nIndex2;
	int				k1,k2;

	//k=frames;
	
	if(axis == 0) //X-Y plane
	{
		if(camera==nCameraHandle1)
		{
			planeDataCCD	=	(WORD*)malloc(nhDim1*nvDim1*sizeof(WORD));
			k=frames;
			nStartPoint=nhDim1*nvDim1*k;
			memcpy(planeDataCCD, g_pRawData+nStartPoint, nhDim1*nvDim1*sizeof(WORD));		
			for(i=0;i<nhDim1*nvDim1;i++)
				planeDataCCD[i]=planeDataCCD[i]-BiasDark1[i];
			
			mean=DataMean(nhDim1,nvDim1, planeDataCCD);
			stdev=DataStdev(nhDim1,nvDim1,planeDataCCD, mean);

			GetMinMax(planeDataCCD,nhDim1*nvDim1,&(*nMinValue),&(*nMaxValue));

			if(*nMinValue>(int)(mean-3*stdev))
				k1=(*nMinValue);
			else
				k1=(int)(mean-3*stdev);
			if(*nMaxValue<(int)(mean+3*stdev))
				k2=(*nMaxValue);
			else
				k2=(int)(mean+3*stdev);
				
			ByteDisplay(nhDim1, nvDim1, planeDataCCD,k1,k2);      				
			
		}
	
	}
	

	if(axis == 1) //Z-Y plane
	{

		if(camera==nCameraHandle1)
		{
			planeDataCCD		=	(WORD*)malloc(g_nFrameNumber1*nvDim1*sizeof(WORD));
			k=frames;
		
			for(j=0; j<nvDim1; j++)
	 		{
				for(i=0; i<g_nFrameNumber1; i++)
		 		{
					nIndex1=nhDim1*nvDim1*i+nhDim1*j+k;
					nIndex2=g_nFrameNumber1*j+i;
					memcpy(planeDataCCD+nIndex2, g_pRawData+nIndex1, sizeof(WORD));
		  		}
	 		}
			
			mean=DataMean(g_nFrameNumber1, nvDim1, planeDataCCD);
			stdev=DataStdev(g_nFrameNumber1, nvDim1, planeDataCCD, mean);
	
			GetMinMax(planeDataCCD,g_nFrameNumber1*nvDim1,&(*nMinValue),&(*nMaxValue));
			
			if(*nMinValue>(int)(mean-3*stdev))
				k1=(*nMinValue);
			else
				k1=(int)(mean-3*stdev);
			if(*nMaxValue<(int)(mean+3*stdev))
				k2=(*nMaxValue);
			else
				k2=(int)(mean+3*stdev);						
			
			ByteDisplay(g_nFrameNumber1,nvDim1,planeDataCCD,k1, k2);
		}
		
		
	} 
}	



void GetMinMax(void* data, int length,  unsigned short* nMinValue, unsigned short* nMaxValue)
{
	unsigned short* RealData;
	int i;   
	
	
	*nMinValue=USHRT_MAX;
	*nMaxValue=0;

	RealData= (unsigned short*)data;

	for(i=0;i<length;i++)
	{
		(*nMaxValue) =( (*nMaxValue)>RealData[i])?(*nMaxValue):RealData[i];
		(*nMinValue) =( (*nMinValue)<RealData[i])?(*nMinValue):RealData[i];
	}
}


float DataMean(int axis1, int axis2, void* pData)
{ 
	 int i;
     float mean=0;
	 WORD* Data=(WORD*)pData;

	 for (i=0;i<axis1*axis2; i++)
	 {
		 mean+=Data[i];
	 }
	 mean/=(axis1*axis2);
	 
	 return mean;
}


float DataStdev(int axis1, int axis2, void* pData, float mean)
{ 
	int i;
	float std=0, tmp; 
	WORD* Data=(WORD*)pData;

    for(i=0;i<axis1*axis2;i++)
	{
		tmp=(float)Data[i]-mean;
		std+= tmp*tmp;
	}
	 
	std /= axis1*axis2;
	std = sqrt(std);
	return std;
}

void marktime_start(char* timestring)
{
	time_t t1;
	struct tm *systime1;

	t1=time(NULL);
	systime1=gmtime(&t1);
			
	sprintf(timestring,"%.2d.%.2d.%.2d-%.2d:%.2d:%.2d",(*systime1).tm_year+1900,(*systime1).tm_mon+1,(*systime1).tm_mday,
				(*systime1).tm_hour,(*systime1).tm_min,(*systime1).tm_sec);
	QueryPerformanceCounter(&gliStart);	
}

void marktime_end(char* timestring)
{
	time_t t2;
	struct tm *systime2;
	
	t2=time(NULL);
	systime2=gmtime(&t2);
	
	sprintf(timestring,"%.2d.%.2d.%.2d-%.2d:%.2d:%.2d",(*systime2).tm_year+1900,(*systime2).tm_mon+1,(*systime2).tm_mday, 
				(*systime2).tm_hour,(*systime2).tm_min,(*systime2).tm_sec);								
	QueryPerformanceCounter(&gliEnd);	
}

float time_difference()
{
	float timediff;
	timediff=(float)(((double)gliEnd.QuadPart-(double)gliStart.QuadPart)/(double)(gliFreq.QuadPart));		
	return timediff;
}


int makefolderscript(char* path)
{
	FILE* fpt;
	
	if(NULL==(fpt=fopen("makefolder.bat","w")))
	{
		MessagePopup("info","Fail to write file\n");
		fclose(fpt);
		return 0;
	}
	else
	{
		fputs("mkdir ",fpt);
		fputs(path,fpt);
		fclose(fpt);
		return 1;
	}	
}	



int TakingFlat(void)
{
	
	char 			temp[50],temp1[50],temp2[50],temp3[50],temp4[50]; 
	int				shift[7]={-110,-90,-30,0,20,80,115}; 
	float diff_time1,diff_time2; 
	int				ii,j,k,startpoint1,startpoint2;
	int				flatstart1,flatstart2;
	int				ccd_acqmode;  
	int attr_top, attr_width, attr_left; 
	
	if((CCD_status!=1) || (Scanner_status!=1))return -1;
	Run_thread_stop=1;
	Lan_Run_thread_stop(1);
	if ((GraphicPanel = LoadPanel (0, "FISS.uir", GRAPHICPNL)) < 0)
								return -1;
	GetPanelAttribute(mainPanel, ATTR_TOP, &attr_top);
	GetPanelAttribute(mainPanel, ATTR_WIDTH, &attr_width);
	GetPanelAttribute(mainPanel, ATTR_LEFT, &attr_left);
	
	SetPanelAttribute(GraphicPanel, ATTR_TOP, attr_top); 
	SetPanelAttribute(GraphicPanel, ATTR_LEFT, attr_left+attr_width+7);
	
	
	
	DisplayPanel(GraphicPanel); 
	
	SetCtrlAttribute(mainPanel, PANEL_viewCCD1,ATTR_DIMMED,1);
	SetCtrlAttribute(mainPanel, PANEL_viewCCD2,ATTR_DIMMED,1);						

	GetPanelHandleFromTabPage(FISSpanelSetting, SetFISS_setTAB, 0, &TabPanelHandle);
	GetCtrlVal(TabPanelHandle, setCCDTAB_CCD_setAcq, &ccd_acqmode);
			
	if(!QueryPerformanceFrequency(&gliFreq))
		MessagePopup("Error","Cannot support the timer of high resolution");
			
	SetCtrlAttribute(mainPanel, PANEL_viewCCD1,ATTR_DIMMED,0);
	SetCtrlAttribute(mainPanel, PANEL_viewCCD2,ATTR_DIMMED,0);		
				
	
	
	
	
	for(ii=0;ii<7;ii++)			   //number of moving grating
	{
			 
			Gratingtracking(2, Data[0].Step+shift[ii]); 
			sprintf(temp, "%d th Grating Position : %d\n", ii+1, Data[0].Step+shift[ii]);
			SetCtrlVal(mainPanel, PANEL_TB_Text_Monitor, temp);
							
			sprintf(temp,"%d",ii+1);
			SetCtrlVal(mainPanel, PANEL_S1, temp);
			fiss_take_mode=1;
			Lan_fiss_take_mode(1);
			memset(g_pRawData  ,0,nhDim1*nvDim1*g_nFrameNumber1*sizeof(WORD));  
			Lan_memset_RawData2(nhDim2*nvDim2*g_nFrameNumber1*sizeof(WORD));  
			memset(flatdata1,0,nhDim1*nvDim1*sizeof(float));
			Lan_memset_flatdata2(nhDim2*nvDim2*sizeof(float));
			
							//Gratingtracking(2, Data[0].Step+shift[ii]);  //move motor at specific angle by using absolute position value(2) 
							
			do
			{
			}while(!((scanpos<1) && (scanpos>-1) && (Scanner_InPosition==1))) ;

			marktime_start(temp1);
			SetCurrentCamera(nCameraHandle1);
			ExecuteProgram(nCameraHandle1,1);
			marktime_end(temp2);
			diff_time1=time_difference();
					
			Lan_Flat_Calc(ii);
			for(k=0;k<g_nFrameNumber1;k++) //number of frame
			{
				startpoint1=nhDim1*nvDim1*k;
				for(j=0;j<nhDim1*nvDim1;j++)
					flatdata1[j]=flatdata1[j]+g_pRawData[j+startpoint1];
									
			}
							
			flatstart1=nhDim1*nvDim1*ii;
							
			for(k=0;k<nhDim1*nvDim1;k++)
			{
				flatdata1[k]=flatdata1[k]/(float)g_nFrameNumber1-(float)BiasDark1[k];
				flatcube1[k+flatstart1]=flatdata1[k];
			}
							 
			DisplayCCDImage(nhDim1, nvDim1, g_nFrameNumber1, nCameraHandle1);	
	}
						
	SaveFlatFits(nhDim1, nvDim1, 7, g_fExpTime1, nCameraHandle1, temp1, temp2, diff_time1,entire_directory); 
	Lan_SaveFlatFits(7, temp1, temp2);
	Gratingtracking(2, Data[0].Step); 	   //return original position
	

	DisplayCCDImage(nhDim1, nvDim1, 7, nCameraHandle1);
	Lan_DisplayCCDImage(7);
	DataSource=1; 
	SetCtrlAttribute(mainPanel, PANEL_viewCCD1,ATTR_DIMMED,0);
	SetCtrlAttribute(mainPanel, PANEL_viewCCD2,ATTR_DIMMED,0);
	SetCtrlVal(mainPanel, PANEL_viewCCD1,1);	
	
	return 1;
}
